/* --------------------------------------------------------------  */
/* (C)Copyright 2006, 2007                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* Redistributions of source code must retain the above copyright  */
/* notice, this list of conditions and the following disclaimer.   */
/*                                                                 */
/* Redistributions in binary form must reproduce the above         */
/* copyright notice, this list of conditions and the following     */
/* disclaimer in the documentation and/or other materials          */
/* provided with the distribution.                                 */
/*                                                                 */
/* Neither the name of IBM Corporation nor the names of its        */
/* contributors may be used to endorse or promote products         */
/* derived from this software without specific prior written       */
/* permission.                                                     */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */

#ifndef __ALF_CONFIG_H_INC__
#define __ALF_CONFIG_H_INC__


//********************************************************
// Platform Definitions
//********************************************************

#ifdef _ALF_PLATFORM_HYBRID_

#if defined(_ALF_PLATFORM_HYBRID_) && (_ALF_PLATFORM_CELL_)
  #error "Cannot configure for both Hybrid ALF and Cell ALF simulatanously"
#endif

#define _ALF_PLATFORM_            hybrid        // this is the architecture dependent include/source path
#ifndef _ALF_PLATFORM_STRING
#define _ALF_PLATFORM_STRING      "hybrid"      // this is the architecture dependent string
#endif
#else
#define _ALF_PLATFORM_CELL_
#define _ALF_PLATFORM_            cell          // this is the architecture dependent include/source path
#ifndef _ALF_PLATFORM_STRING
#define _ALF_PLATFORM_STRING      "cell"        // this is the architecture dependent string
#endif
#endif
//********************************************************
// Conditional Definitions
//********************************************************

#define ADD_WATCH_WQ_POLICY     
//#define ADD_ASYNC_WQ_POLICY     


#define _ALF_DEBUG_LEVEL_           0 //  0 to 10,
                                       //  0 is no debug print info at all. 
                                       //  10 is full info
/**
  _ALF_DEBUG_LEVEL_ = 1           DUMP PPU ERROR INFORMATION
  _ALF_DEBUG_LEVEL_ = 2           DUMP SPU ERROR INFORMATION
  _ALF_DEBUG_LEVEL_ = 3           DUMP PIPELINE INFORMATION
  _ALF_DEBUG_LEVEL_ = 4           DUMP DMA INFORMATION
 */

//--------------------------------------------------------
//  Error condition check, choose only one of them
//--------------------------------------------------------
//#define _ALF_CFG_CHECK_STRICT_      //do strict error and parameter check, may loose 
                                    //performance, usually for application tuning
//#define _ALF_CFG_CHECK_LOOSE_     //do loose check and optimize for performance,
                                    //usually for final release of app
//#define _ALF_CFG_CHECK_DEBUG_     // do debug check and dump errors if you want



//********************************************************
// Constants 
//********************************************************

// the following macros will automatically determin the following macros
// _ALF_GNUC_   // we are compiling with GNUC
// _ALF_XLC_    // we are compiling with IBM XLC
// _ALF_X86_64_ // we are compiling for X86_64
// _ALF_PPE_    // we are compiling for PPE
// _ALF_SPE_    // we are compiling for SPE


#if !defined(__GNUC__) && !defined(__IBMC__)
    #error "Your compiler is not supported yet !"
#endif

#if defined(__GNUC__)
    #if (__GNUC__ < 4) || ( !defined(__PPC__) && !defined(__SPU__) && !defined(__i386__) && !defined(__x86_64__)) || ( defined(__PPC__) && defined(__SPU__) && defined(__i386__) && defined(__x86_64__))
//    #if (__GNUC__ < 4) || ( !defined(__PPC__) && !defined(__SPU__) ) || ( defined(__PPC__) && defined(__SPU__) )
        #error "Your GCC compiler is not supported !"
    #else
        #define alf_likely(_c) __builtin_expect((_c), 1)
        #define alf_unlikely(_c) __builtin_expect((_c), 0)
        #define _ALF_GNUC_  1
        #ifdef __PPC__
            #define _ALF_PPE_   1
            #ifdef __PPC64__
                #define _ALF_64B_   1
            #else
                #define _ALF_32B_   1
            #endif
        #elif __SPU__
            #define _ALF_SPE_   1
            #define _ALF_32B_   1
        #elif __i386__
            #define _ALF_X86_   1
            #define _ALF_32B_   1
        #elif __x86_64__
            #define _ALF_X86_   1
            #define _ALF_64B_   1
        #endif
    #endif
#endif


#if defined(__IBMC__)
    #if (__IBMC__ < 800)
        #error "Your XLC compiler is not supported !"
    #else
        #define _ALF_XLC_  1
#ifndef alf_likely
        #define alf_likely(_c) (_c)
#endif        
#ifndef alf_unlikely
        #define alf_unlikely(_c) (_c)
#endif        
        #ifdef __PPC__
            #define _ALF_PPE_   1
            #ifdef __PPC64__
                #define _ALF_64B_   1
            #else
                #define _ALF_32B_   1
            #endif
        #else
            #define _ALF_SPE_   1
            #define _ALF_32B_   1
        #endif
    #endif
#endif

#ifdef _ALF_32B_ //32Bit Compiler

  #define PTR_TO_ADDR64( ptr , addr) {                                  \
    (addr) = ( (unsigned long)(ptr) & 0xFFFFFFFFUL);      \
  }

  //! Macro to cast 64 bits effective address to a pointer
  #define ADDR64_TO_PTR( addr, type, ptr) { (ptr) = (type)((unsigned int)addr); }

  #define ADDR64_PTR_OFFSET(addr, offset) (addr.ui[1] + offset)

#elif _ALF_64B_ //64Bit Compiler

  #define PTR_TO_ADDR64( ptr , addr) {                                  \
    (addr) = (unsigned long long)(ptr);                             \
  }

  //! Macro to cast 64 bits effective address to a pointer
  #define ADDR64_TO_PTR( addr, type, ptr) { (ptr) = (type)(addr);}
  #define ADDR64_PTR_OFFSET(addr, offset) (addr + offset)

#elif //Unsupported compiler
  #error "Your compiler is not supported yet !"
#endif


//********************************************************
// Data Structures
//********************************************************


//********************************************************
// APIs
//********************************************************



#endif // __ALF_CONFIG_H_INC__

