/* --------------------------------------------------------------  */
/* (C)Copyright 2007                                               */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* Redistributions of source code must retain the above copyright  */
/* notice, this list of conditions and the following disclaimer.   */
/*                                                                 */
/* Redistributions in binary form must reproduce the above         */
/* copyright notice, this list of conditions and the following     */
/* disclaimer in the documentation and/or other materials          */
/* provided with the distribution.                                 */
/*                                                                 */
/* Neither the name of IBM Corporation nor the names of its        */
/* contributors may be used to endorse or promote products         */
/* derived from this software without specific prior written       */
/* permission.                                                     */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */

#ifndef __ALF_MISC_CBEA_H_INC__
#define __ALF_MISC_CBEA_H_INC__

//********************************************************
// Constants
//********************************************************

//--------------------------------------------------------
// Data Aligment Macros
//--------------------------------------------------------

#ifndef _ALF_PACKED_
#if defined(_ALF_XLC_) || defined(_ALF_GNUC_)
#define _ALF_PACKED_(_alfal_var_def_)   _alfal_var_def_  __attribute__((__packed__))
#define _ALF_ALIGN_(_alfal_var_def_, _alfal_al_)  _alfal_var_def_  __attribute__((__aligned__(_alfal_al_)))
#else
    // fixme:  add support for your compiler
#define _ALF_PACKED_(_alfal_var_def_)   _alfal_var_def_
#define _ALF_ALIGN_(_alfal_var_def_, _alfal_al_)  _alfal_var_def_
#endif
#endif

#ifndef _ALF_SIZE_ALIGN_
#define _ALF_SIZE_ALIGN_(len, align) (((len) + ((1 << (align)) - 1)) & (~((1 << (align)) - 1)))
#endif


//--------------------------------------------------------
// Data Aligment Macros
//--------------------------------------------------------
//! Macro to align size to 16 bytes which is required for simd instruction
#define _ALF_SIMD_SIZE_ALIGN_(x) _ALF_SIZE_ALIGN_(x, 4)

//! Macro to align size to 128 bytes which can provide most efficient dma data transfer
#define _ALF_DMA_SIZE_ALIGN_(x)  _ALF_SIZE_ALIGN_(x, 7)

//! Macro to declare a variable which will be used as dma transfer source or destination
#define _ALF_DECLARE_DMA_(_type_, _var_)  _ALF_ALIGN_(volatile _type_ _var_, 128)

//! Macro to declare a variable which will be accessed by simd instructions
#define _ALF_DECLARE_SIMD_(_type_, _var_) _ALF_ALIGN_(_type_ _var_, 16)

//! Macro to verify whether the address or size satisfy the specific alignment requirement 
#define _ALF_IS_ALIGNED_(_add_, n)   (((unsigned int)(_add_)&((1 << (n)) - 1)) == 0)

//! Macro to verify whether the address or size satisfy the simd alignment requirement 
#define _ALF_IS_SIMD_ALIGNED_(_add_) (((unsigned int)(_add_) & 0xf) == 0)

//! Macro to verify whether the address or size can provide most efficient dma transfer 
#define _ALF_IS_DMA_ALIGNED_(_add_) (((unsigned int)(_add_) & 127) == 0)




//! Define size of a single element with a specific data type
static const int g_size_of_type[] = {
  1,                            // size of ALF_DATA_BYTE
  2,                            // size of ALF_DATA_INT16
  4,                            // size of ALF_DATA_INT32
  8,                            // size of ALF_DATA_INT64
  4,                            // size of ALF_DATA_FLOAT
  8,                            // size of ALF_DATA_DOUBLE
};

//! Retrieve size of a single element with a specific data type
#define ALF_GET_SIZE_OF_TYPE( _type_ )                        \
(((int)(_type_) >= ALF_DATA_BYTE && (int)(_type_) <= ALF_DATA_DOUBLE ) ? \
  g_size_of_type[(_type_)] : 0)



//! Macro to verify whether a dma size is valid
#define IS_VALID_DMA_SIZE(_dma_size_)   \
(((_dma_size_) == 1 || (_dma_size_) == 2 || (_dma_size_) == 4 || (_dma_size_) == 8 || _ALF_IS_SIMD_ALIGNED_(_dma_size_)))

#endif /* __ALF_MISC_CBEA_H_INC__ */
